@@ -27,7 +27,7 @@
 *
 ********************************************************************************
 *
-*  $Id: psad_funcs.c 1901 2006-12-27 05:18:08Z mbr $
+*  $Id: psad_funcs.c 1959 2007-02-13 00:33:15Z mbr $
 */
 
 /* includes */
@@ -109,7 +109,7 @@ void write_pid(const char *pid_file, const pid_t pid)
     return;
 }
 
-int find_char_var(char *search_str, char *charvar, char *line)
+int find_char_var(const char *search_str, char *charvar, char *line)
 {
     char *index_tmp;
     int char_ctr = 0, i;
@@ -166,6 +166,17 @@ int find_char_var(char *search_str, char *charvar, char *line)
     return 1;
 }
 
+void *safe_malloc(const unsigned int len)
+{
+    void *buf = NULL;
+    buf = malloc(len);
+    if (buf == NULL) {
+        fprintf(stderr, "[*] Could not malloc() %d bytes\n", len);
+        exit(EXIT_FAILURE);
+    }
+    return buf;
+}
+
 int has_sub_var(char *var_name, char *value, char *sub_var,
     char *pre_str, char *post_str)
 {
